#!/usr/bin/perl
################################################################################
# eWLA (e Web Log Analyzer)                         Version 0.1                #
# Copyright 2000 Frederic TYNDIUK (FTLS)            All Rights Reserved.       #
# E-Mail: tyndiuk@ftls.org                          Script License: GPL        #
# Created  05/31/00                                 Last Modified 05/25/00     #
# Scripts Archive at:                               http://www.ftls.org/cgi/   #
################################################################################
# Function : Creates multi-page HTML reports of Apache log statistics.         #
#   Log analysers part.                                                        #
################################################################################
########################## license & copyright header ##########################
#                                                                              #
#                     Copyright (c) 2000 by TYNDIUK Frederic                   #
#                                                                              #
#       This program is free software; you can redistribute it and/or          #
#       modify it under the terms of the GNU General Public License as         #
#       published by the Free Software Foundation; either version 2 of         #
#       the License, or (at your option) any later version.                    #
#                                                                              #
#       This program is distributed in the hope that it will be useful,        #
#       but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#       GNU General Public License for more details.                           #
#                                                                              #
#       You should have received a copy of the GNU General Public License      #
#       along with this program in the file 'COPYING'; if not, write to        #
#       the Free Software Foundation, Inc., 59 Temple Place - Suite 330,       #
#       Boston, MA 02111-1307, USA, or contact the author:                     #
#                                                                              #
#                                   TYNDIUK Frederic <tyndiuk@ftls.org>        #
#                                            <http://www.ftls.org/>            #
#                                                                              #
######################## end license & copyright header ########################
################################################################################
# Optionals  Variables:                                                        #

use strict;

require 'ewla_lib.pl';

my %params; $params{'ConfigFile'} = 'ewla.cfg';

# Nothing Below this line needs to be altered!                                 #
################################################################################
#    DO NOT EDIT ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU'RE DOING    #
################################################################################
my $COPYRIGHT  = "eWLA v0.1 (C) 2000 Frederic TYNDIUK (alias FTLS)\n";
   $COPYRIGHT .= "Report bugs to tyndiuk\@ftls.org, News and Updates: http://www.ftls.org/\n\n";

my $VERBOSE = 1;
my $DEBUG = 0;
my $SESSION_DELAY = 30;

my $IMG_EXT = '(gif|jpg|jpeg|png|ico)';
my $DOWNLOAD_EXT = '(tar|gz|bzip2|zip|exe|pdf|rpm|deb)';
my %MONTH_TO_NUM = ( 'Jan' => '01', 'Feb' => '02', 'Mar' => '03',
					 'Apr' => '04', 'May' => '05', 'Jun' => '06',
					 'Jul' => '07', 'Aug' => '08', 'Sep' => '09',
					 'Oct' => '10', 'Nov' => '11', 'Dec' => '12');

my (%empty, %resolved_ip, %trace_visit, %test_visit);
my (%all_hits_year, %all_bytes_year, %all_pages_views_year, %all_visits_year,
	%all_hits_monthly, %all_bytes_monthly, %all_pages_views_monthly, %all_visits_monthly,
	%all_hits_daily, %all_bytes_daily, %all_pages_views_daily, %all_visits_daily,
	%all_hits_dayofweek, %all_bytes_dayofweek, %all_visits_dayofweek, %all_pages_views_dayofweek,
	%all_hits_hourly, %all_bytes_hourly, %all_pages_views_hourly, %all_visits_hourly,
	%files_hits_daily, %files_bytes_daily, %files_visits_daily,
	%domains_hits_daily, %domains_bytes_daily, %domains_visits_daily, %domains_pages_views_daily,
	%users_hits_daily, %users_bytes_daily, %users_visits_daily, %users_pages_views_daily,
	%referrer_hits_daily, %referrer_visits_daily, %referrer_pages_views_daily,
	%agent_hits_daily, %agent_visits_daily, %agent_pages_views_daily,
	%files_error_daily, %adverts_hits_daily, %seeker, %record);

my ($nb_log_line_total, $nb_log_line_analysed);
my ($adv_img_txt, $resumes_dir);
my ($date_end_last_report, $last_log_today, $first_log_today);
my ($analyse, $month_resume, $year_resume, $file_resume);


########################################
# Read args and config file            #
########################################

%params = &read_args(%params);

%params = &load_data_from_config_file(%params);

########################################
# Analyser                             #
########################################
print $COPYRIGHT;
print "Log Analysis for $params{'ServerName'}\n" if ($VERBOSE);

$resumes_dir = $params{'ResumeDir'};
if ($params{'HideReferrer'}) { $params{'HideReferrer'} = 'ExcludeThisPath' }

#Load last resume date
if ((-e $resumes_dir."date.res") && $params{'LogOnlyNew'}) {
	open (RESUME, $resumes_dir."date.res") || die "  Error opening resume file: $resumes_dir/date.res, error: $!\n";
	$date_end_last_report = <RESUME>;
	close(RESUME);
	chomp ($date_end_last_report);
}

print "  Open log file: $params{'LogFile'}\n" if ($VERBOSE);

$adv_img_txt = $params{'AdvImg'};
$adv_img_txt =~ s/\(|\)//;
$adv_img_txt = join(', ', split(/\s*\|\s*/, $adv_img_txt));

$nb_log_line_total = $nb_log_line_analysed = 0;
$analyse = 0;

my $visit = "";
if (1) {
# Open and analysis log file
my ($date_today, $old_file_resume, $referrer, $domain, $is_page_view, $is_new_visitor, $is_img, $is_adv_img, $exclude, $is_error, $no_visitor);
open (LOGFILE,"<$params{'LogFile'}") || die "  Error opening log file: $params{'LogFile'}, error: $!\n";
while(<LOGFILE>) {
	$nb_log_line_total++;
	s/\n//g; s/\s+/ /go;
	my ($a_rhost, $a_ruser, $a_date, $a_time, $a_file, $a_status, $a_bytes, $a_referrer, $a_agent) = &load_log_line($_);

	my ($a_day, $a_month, $a_year)	= split(/\//, $a_date);
	my ($a_hour, $a_min, $a_sec) = split(/:/, $a_time);
	$date_today = $a_date;

	$last_log_today = "$a_date $a_time";
	$file_resume = "d".$a_year."_".$a_month."_".$a_day.".res";

	if ($analyse || &date_is_sup("$a_date $a_time", $date_end_last_report)) {
		$analyse = 1;
		if ($first_log_today eq '') {
			$first_log_today = "$a_year/$a_month/$a_day";
		}
		$nb_log_line_analysed++;
		if ($file_resume ne $old_file_resume) {
			if ($old_file_resume =~ /_/) { 
				&save_resume_file($resumes_dir.$old_file_resume, $date_today);
				&init_resume_value;
			}
			print "    Analyse $a_day/$a_month/$a_year day\n" if ($VERBOSE);
			$old_file_resume = $file_resume;
			if (-e $resumes_dir.$file_resume) {
				&init_resume_value;
				$no_visitor = 0;
				&load_resume_file($resumes_dir.$file_resume);
			}
		}

		$exclude = 0;
		if ($a_file !~ /$params{'HideReferrer'}/) {
			$domain = &try_resolv_ip($a_rhost);
			$domain = &sup_sub_domains($domain, 'xxx.dom');
			$referrer = $a_referrer;
			$referrer =~ s/\?.+//g;

			$is_error = (! ((($a_status > 199) && ($a_status < 300)) || ($a_status == 304)));
			$is_img = ($a_file =~ /\.$IMG_EXT/i);
			$is_adv_img = ($a_file =~ /$params{'AdvImg'}/i);
			$is_page_view = ((! $is_img) && ($a_file !~ /\.$DOWNLOAD_EXT/i));

			if (! $is_img) {
				$is_new_visitor = 0;
				if (! ((($a_min + $a_hour * 60) - $test_visit{$a_rhost.$a_agent} <= $SESSION_DELAY - 2) && ($a_file !~ /\/$params{'HideReferrer'}/))) {
					$is_new_visitor = 1;
					$no_visitor++;
					if ($trace_visit{$a_rhost.$a_agent} ne "") {
						$trace_visit{$a_rhost.$a_agent.$no_visitor} = $trace_visit{$a_rhost.$a_agent};
						$trace_visit{$a_rhost.$a_agent} = "";
					}
				}
				$test_visit{$a_rhost.$a_agent} = $a_min + $a_hour * 60;
			} else { $is_new_visitor = 0; }

			if ($is_page_view) { $trace_visit{$a_rhost.$a_agent} .= "\t$a_file"; }

			if ($is_page_view && (! $is_error)) {
				$all_pages_views_hourly{$a_hour}++;
				$all_pages_views_daily{'00'}++;
				$domains_pages_views_daily{$domain}++;
			}	

			if (($is_new_visitor) && (! $is_img)) {
				$all_visits_hourly{$a_hour}++;
				$all_visits_daily{'00'}++;
				$domains_visits_daily{$domain}++;
				if (! $is_error) { $files_visits_daily{$a_file}++; }
			}

			my $sv_file = $a_file;
			if ($is_adv_img) {
				my $r = $referrer;
				$r =~ s/\/.*\.(.*html*)$/\//;
				my $file = $a_file;
				$file =~ /.*\//;
				$adverts_hits_daily{"$file|$r"}++;
				$a_file = "Adverts Images Files";
				$exclude = 1
			} elsif ($is_img) {
				$a_file = "Images Files";
				$exclude = 1;
			}

			if (! $is_error) {
				$files_hits_daily{$a_file}++;
				$files_bytes_daily{$a_file} += $a_bytes;
			}

			$all_hits_hourly{$a_hour}++;
			$all_hits_daily{'00'}++;

			$all_bytes_hourly{$a_hour} += $a_bytes;
			$all_bytes_daily{'00'} += $a_bytes;

			if (! $exclude) { 
				$domains_hits_daily{$domain}++; 
				$domains_bytes_daily{$domain} += $a_bytes;
			}

			if (($a_referrer =~ /\w/) && (! $is_img) && (! $is_error)) {
				&extract_seeker_keywords($a_referrer); 
				$referrer_hits_daily{"$a_file|$referrer"}++;
				if ($is_new_visitor) { $referrer_visits_daily{"$a_file|$referrer"}++; }
				if ($is_page_view)   { $referrer_pages_views_daily{"$a_file|$referrer"}++; }
			}

			if ($a_agent =~ /\w/) {
				$agent_hits_daily{$a_agent}++;
				if ($is_new_visitor) { $agent_visits_daily{$a_agent}++ }
				if ($is_page_view)   { $agent_pages_views_daily{$a_agent}++ }
			}
			if ($a_ruser =~ /\w/) {
				$users_hits_daily{$a_ruser}++;
				$users_bytes_daily{$a_ruser} += $a_bytes;
				if ($is_new_visitor) { $users_visits_daily{$a_ruser}++; }
				if ($is_page_view)   { $users_pages_views_daily{$a_ruser}++; }
			}
			if ($is_error) {
				if ($a_status >= 400) {
					$files_error_daily{"$a_status|$sv_file"}++;
				}
			}
		} # end if Not Hide
	} # end if analyse
} # end while

if ($nb_log_line_analysed > 0)  {
	&save_resume_file($resumes_dir.$file_resume, $date_today);
	open (RESUME, ">$resumes_dir"."date.res") || die "  Error write resume date file: $resumes_dir date.res, error: $!\n";
	print RESUME $last_log_today;
	close(RESUME);
}

}
#$nb_log_line_analysed = 1;
#$first_log_today = "2000/03/26";
########################################
# Monthly and Yearly Analysis          #
########################################
my ($today_year, $today_month, $today_day, %record_y);
my ($list_day, $list_month, $month, $day, $year, $date, $month_old, $year_old, $file_name, $val);
print "    $nb_log_line_analysed of $nb_log_line_total log entries processed\n" if ($VERBOSE);

if ($nb_log_line_analysed > 0) {
	print "  Generating Monthy resume file\n";
		&init_resume_value;
		$month_resume = 1;
		($today_year, $today_month, $today_day) = split(/\//, $first_log_today);

		$list_day = $resumes_dir."d".$today_year."_";
		foreach(<$list_day*>) {
			$file_name = $_;
			s/$resumes_dir(d)$today_year\_(\d\d)_(\d\d)//;
			$month = $2;
			$day = $3;
			$date = "$day/$month/$today_year";
			if ($month >= $today_month) {
				if ($month_old ne $month) {
					if ($month_old ne '') {
						&save_resume_file($resumes_dir."m".$today_year."_".$month_old.".res", '');
						&init_resume_value;
						$all_hits_monthly{'00'}   = $all_bytes_monthly{'00'} = "";
						$all_visits_monthly{'00'} = $all_pages_views_monthly{'00'} = "";
					}
					$month_old = $month;
				}
				my $dayofweek = &date2wkday($month, $day, $year);
				#$all_hits_monthly{'00'}   = $all_bytes_monthly{'00'} = "";
				#$all_visits_monthly{'00'} = $all_pages_views_monthly{'00'} = "";
				$all_hits_daily{'00'}   = $all_bytes_daily{'00'} = "";
				$all_visits_daily{'00'} = $all_pages_views_daily{'00'} = "";
				&load_resume_file($file_name);
				#print "$day $month $month_old load $file_name\n";
				$all_hits_dayofweek{$dayofweek}   += $all_hits_daily{'00'};
				$all_bytes_dayofweek{$dayofweek}  += $all_bytes_daily{'00'};
				$all_visits_dayofweek{$dayofweek} += $all_visits_daily{'00'};
				$all_pages_views_dayofweek{$dayofweek}  += $all_pages_views_daily{'00'};
				$all_hits_daily{$day}   = $all_hits_daily{'00'};
				$all_bytes_daily{$day}  = $all_bytes_daily{'00'};
				$all_visits_daily{$day} = $all_visits_daily{'00'};
				$all_pages_views_daily{$day} = $all_pages_views_daily{'00'};
				$all_hits_monthly{'00'}   += $all_hits_daily{'00'};
				$all_bytes_monthly{'00'}  += $all_bytes_daily{'00'};
				$all_visits_monthly{'00'} += $all_visits_daily{'00'};
				$all_pages_views_monthly{'00'} += $all_pages_views_daily{'00'};

				($val) = split(/\|/, $record{'hits'});
				if ($val <= $all_hits_daily{'00'}) { $record{'hits'} = "$all_hits_daily{'00'}|$date" }
				($val) = split(/\|/, $record{'bytes'});
				if ($val <= $all_bytes_daily{'00'}) { $record{'bytes'} = "$all_bytes_daily{'00'}|$date"; }
				($val) = split(/\|/, $record{'pgv'});
				if ($val <= $all_pages_views_daily{'00'}) { $record{'pgv'} = "$all_pages_views_daily{'00'}|$date"; }
				($val) = split(/\|/, $record{'visits'});
				if ($val <= $all_visits_daily{'00'}) { $record{'visits'} = "$all_visits_daily{'00'}|$date"; }
			}
		}
		$all_hits_daily{'00'}   = $all_bytes_daily{'00'} = "";
		$all_visits_daily{'00'} = $all_pages_views_daily{'00'} = "";
		&save_resume_file($resumes_dir."m".$today_year."_".$month.".res", '');


	print "  Generating Year resume file\n";
		my($valy, $datey);
		$year_resume = 1;
		&init_resume_value;
		($today_year, $today_month, $today_day) = split(/\//, $first_log_today);

		$list_month = $resumes_dir."m";
		foreach(<$list_month*>) {
			$file_name = $_;
			s/$resumes_dir(m)(\d+)_(\d\d)//;
			$year = $2;
			$month = $3;
			if ($year_old ne $year) {
				if ($year_old ne '') {
					%record = %record_y;
					&save_resume_file($resumes_dir."y".$year_old.".res", '');
					&init_resume_value;
				}
				$year_old = $year;
			}
			$all_hits_monthly{'00'}   = $all_bytes_monthly{'00'} = "";
			$all_visits_monthly{'00'} = $all_pages_views_monthly{'00'} = "";
			&load_resume_file($file_name);
			$all_hits_monthly{$month}   = $all_hits_monthly{'00'};
			$all_bytes_monthly{$month}  = $all_bytes_monthly{'00'};
			$all_visits_monthly{$month} = $all_visits_monthly{'00'};
			$all_pages_views_monthly{$month} = $all_pages_views_monthly{'00'};

			($valy, $datey) = split(/\|/, $record_y{'hits'});
			($val, $date) = split(/\|/, $record{'hits'});
			if ($val >= $valy) { $record_y{'hits'} = "$val|$date"; }
			($valy, $datey) = split(/\|/, $record_y{'bytes'});
			($val, $date) = split(/\|/, $record{'bytes'});
			if ($val >= $valy) { $record_y{'bytes'} = "$val|$date"; }
			($valy, $datey) = split(/\|/, $record_y{'pgv'});
			($val, $date) = split(/\|/, $record{'pgv'});
			if ($val >= $valy) { $record_y{'pgv'} = "$val|$date"; }
			($valy, $datey) = split(/\|/, $record_y{'visits'});
			($val, $date) = split(/\|/, $record{'visits'});
			if ($val >= $valy) { $record_y{'visits'} = "$val|$date"; }
		}
		$all_hits_monthly{'00'}   = $all_bytes_monthly{'00'} = "";
		$all_visits_monthly{'00'} = $all_pages_views_monthly{'00'} = "";
		%record = %record_y;
		&save_resume_file($resumes_dir."y".$year.".res", '');
} # end if($nb_log_line_analysed > 0)

########################################
# Subs                                 #
########################################

sub load_resume_file {
	my($file) = @_;
	print "      Load resume file: '$file'\n" if ($DEBUG);
	open (RESUMEFILE,"<$file") || die "  Error opening resume file: $file, error: $!\n";
	while(<RESUMEFILE>) {
		chomp($_);
		my @col = split(/\|/, $_);
		if (/^hourly/) {
			$all_hits_hourly{"$col[1]"} += $col[2]; 
			$all_bytes_hourly{$col[1]}  += $col[3]; 
			$all_pages_views_hourly{$col[1]} += $col[4]; 
			$all_visits_hourly{$col[1]} += $col[5];
		}
		elsif (/^daily/) {
			$all_hits_daily{$col[1]}  += $col[2]; 
			$all_bytes_daily{$col[1]} += $col[3]; 
			$all_pages_views_daily{$col[1]} += $col[4]; 
			$all_visits_daily{$col[1]}+= $col[5];
		}
		elsif (/^dayofweek/) {
			$all_hits_dayofweek{$col[1]}   += $col[2];
			$all_bytes_dayofweek{$col[1]}  += $col[3];
			$all_pages_views_dayofweek{$col[1]}  += $col[4];
			$all_visits_dayofweek{$col[1]} += $col[5];
		}
		elsif ((/^monthly/) && ($month_resume)) {
			$all_hits_monthly{$col[1]}  += $col[2];
			$all_bytes_monthly{$col[1]} += $col[3]; 
			$all_pages_views_monthly{$col[1]} += $col[4]; 
			$all_visits_monthly{$col[1]}+= $col[5];
		}
		elsif ((/^year/) && ($year_resume)) {
			$all_hits_year{$col[1]}  += $col[2]; 
			$all_bytes_year{$col[1]} += $col[3]; 
			$all_pages_views_year{$col[1]} += $col[4]; 
			$all_visits_year{$col[1]}+= $col[5];
		}
		elsif(/^files/) { 
			$files_hits_daily{$col[1]}  += $col[2]; 
			$files_bytes_daily{$col[1]} += $col[3];
			$files_visits_daily{$col[1]} += $col[5];
		}
		elsif(/^domains/) { 
			$domains_hits_daily{$col[1]}  += $col[2]; 
			$domains_bytes_daily{$col[1]} += $col[3]; 
			$domains_pages_views_daily{$col[1]} += $col[4]; 
			$domains_visits_daily{$col[1]}+= $col[5];
		}
		elsif(/^referrer/) {
			$referrer_hits_daily{"$col[1]|$col[2]"}  += $col[3]; 
			$referrer_pages_views_daily{"$col[1]|$col[2]"} += $col[5]; 
			$referrer_visits_daily{"$col[1]|$col[2]"} += $col[6];
		}
		elsif(/^agent/) {
			$agent_hits_daily{$col[1]}   += $col[2]; 
			$agent_pages_views_daily{$col[1]}  += $col[4]; 
			$agent_visits_daily{$col[1]} += $col[5];
		}
		elsif(/^error/) {
			$files_error_daily{"$col[1]|$col[2]"} += $col[3]; 
		}
		elsif(/^seeker/) {
			$seeker{"$col[1]|$col[2]"} += $col[3];
		}
		elsif(/^adverts/) {
			$adverts_hits_daily{"$col[1]|$col[2]"} += $col[3];
		}
		elsif(/^record/) {
			$record{$col[1]} = "$col[2]|$col[3]";
		}
		elsif(/^trace_visit/) {
			$trace_visit{$col[1]} = $col[2];
		}
		elsif(/^users/) { 
			$users_hits_daily{$col[1]}  += $col[2]; 
			$users_bytes_daily{$col[1]} += $col[3]; 
			$users_pages_views_daily{$col[1]} += $col[4]; 
			$users_visits_daily{$col[1]}+= $col[5];
		}
	}
	close(RESUMEFILE);
}

sub save_resume_file {
	my($file, $a_date) = @_;
	if (! $file) { return };
	my($data, $val, $date, $k, %tv);
	if ((! $month_resume) && (! $year_resume)) {
		foreach $k (keys %trace_visit) {
			$tv{$trace_visit{$k}}++;
		}
	} else {
		%tv = %trace_visit;
	}

	$data .= &sv_htab_value('trace_visit' , \%tv, \%empty, \%empty, \%empty, 2 + $month_resume);
	$data .= &sv_htab_value('hourly' , \%all_hits_hourly, \%all_bytes_hourly, \%all_pages_views_hourly, \%all_visits_hourly, 0);
	$data .= &sv_htab_value('daily'  , \%all_hits_daily, \%all_bytes_daily, \%all_pages_views_daily, \%all_visits_daily, 0);
	if ($month_resume) {
		$data .= &sv_htab_value('monthly', \%all_hits_monthly, \%all_bytes_monthly, \%all_pages_views_monthly, \%all_visits_monthly, 0);
		$data .= &sv_htab_value('dayofweek', \%all_hits_dayofweek, \%all_bytes_dayofweek, \%all_pages_views_dayofweek, \%all_visits_dayofweek, 0);
	}
	if ($year_resume) {
		$data .= &sv_htab_value('year'   , \%all_hits_year, \%all_bytes_year, \%all_pages_views_year, \%all_visits_year, 0);
	}
	$data .= &sv_htab_value('files'  , \%files_hits_daily, \%files_bytes_daily, \%empty, \%files_visits_daily, 1);
	$data .= &sv_htab_value('domains', \%domains_hits_daily, \%domains_bytes_daily, \%domains_pages_views_daily, \%domains_visits_daily, 2);
	$data .= &sv_htab_value('referrer', \%referrer_hits_daily, \%empty, \%referrer_pages_views_daily, \%referrer_visits_daily, 0);
	$data .= &sv_htab_value('agent'  , \%agent_hits_daily, \%empty, \%agent_pages_views_daily, \%agent_visits_daily, 1);
	$data .= &sv_htab_value('error'  , \%files_error_daily, \%empty, \%empty, \%empty, 0);
	$data .= &sv_htab_value('seeker', \%seeker, \%empty, \%empty, \%empty, 0);
	$data .= &sv_htab_value('adverts', \%adverts_hits_daily, \%empty, \%empty, \%empty, 0);
	$data .= &sv_htab_value('users', \%users_hits_daily, \%users_bytes_daily, \%users_pages_views_daily, \%users_visits_daily, 0);

	if ($month_resume || $year_resume) {
		$data .= &sv_htab_value('record', \%record, \%empty, \%empty, \%empty, 0);
	}

	print "      Save resume file: '$file'\n" if ($DEBUG);
	open(SAVE, ">$file") || die "Cannot write resume file ($file), error: $!";
	print SAVE $data;
	close(SAVE);
}

sub sv_htab_value {
	my($txt, $htab1, $htab2, $htab3, $htab4, $noiselevel) = @_;
	my($key, $data);
	foreach $key ( sort(keys (%$htab1)) ) {
		if (($key ne '') && ($$htab1{$key} > $noiselevel)) {
			$data .= $txt."|$key|$$htab1{$key}|$$htab2{$key}|$$htab3{$key}|$$htab4{$key}\n";
			$data =~ s/\|+$//;
		}
	}
	return $data;
}

sub init_resume_value {
	%trace_visit = ();
	%test_visit = ();
	%all_hits_year = ();
	%all_bytes_year = ();
	%all_pages_views_year = ();
	%all_visits_year = ();
	if (! $month_resume) {
		%all_hits_monthly = ();
		%all_bytes_monthly = ();
		%all_pages_views_monthly = ();
		%all_visits_monthly = ();
	}
	%all_hits_dayofweek = ();
	%all_bytes_dayofweek = ();
	%all_visits_dayofweek = ();
	%all_pages_views_dayofweek = ();
	%all_hits_daily = ();
	%all_bytes_daily = ();
	%all_pages_views_daily = ();
	%all_visits_daily = ();
	%all_hits_hourly = ();
	%all_bytes_hourly = ();
	%all_pages_views_hourly = ();
	%all_visits_hourly = ();
	%files_hits_daily = ();
	%files_bytes_daily = ();
	%files_visits_daily = ();
	%domains_hits_daily = ();
	%domains_bytes_daily = ();
	%domains_visits_daily = ();
	%domains_pages_views_daily = ();
	%users_hits_daily = ();
	%users_bytes_daily = ();
	%users_visits_daily = ();
	%users_pages_views_daily = ();
	%referrer_hits_daily = ();
	%referrer_visits_daily = ();
	%referrer_pages_views_daily = ();
	%agent_hits_daily = ();
	%agent_visits_daily = ();
	%agent_pages_views_daily = ();
	%files_error_daily = ();
	%adverts_hits_daily = ();
	%seeker = ();
	%record = ();
}

sub load_log_line {
	# Convert log line into value...
	# Only combined Log type on this version
	($_) = @_;
	my ($a_rhost, $a_ruser, $a_date, $a_time, $a_req, $a_status, $a_bytes, $a_referrer, $a_agent, $a_file, $v);

	# Combined Log type
	if (($a_rhost, $v, $a_ruser, $a_time, $a_req, $a_status, $a_bytes, $a_referrer, $a_agent) 
		= /^(\S+) (\S+) (.+) \[(.+)\] \"(.+)\" (\S+) (\S+) \"(.*)\" \"(.*)\"/o) {
		$a_file = $a_req;
		$a_file =~ s/\w+ //; $a_file =~ s/ .*$//;
		$a_time =~ s/ [+-]\d*$//g;
		($a_date, $a_time) = split (/:/, $a_time, 2);
		$a_date =~ s/(\d+)\/(\w+)\/(\d+)/$1\/$MONTH_TO_NUM{$2}\/$3/;
	}

	# Pre process agent, referrer...
	$a_file =~ s/$params{'DefaultHomePage'}$//i;
	$a_file =~ s/\?.+//g; $a_file  =~ s/\.\///g;
	$a_file =~ s/%7E/~/g;
	$a_file =~ s/http:\/\/[\w\.-]+\//\//;

	if (($a_referrer !~ /https*:|bookmarks/) || ($a_referrer =~ /https*:\/\/[\w\.-]*$params{'HostNames'}/i)) {
		$a_referrer = "";
	}
	$a_referrer =~ s/(https*:\/\/)www\./$1/;

	if (($a_agent =~ /\w/) && ($a_agent !~/ip3000\.com/)) {
		$a_agent =~ s/_/ /g;
		$a_agent =~ s/ Windows 9(5|8); .*/ Windows 9$1)/i;
		$a_agent =~ s/ (I|U) *;/ /i;    # sup I|U;
		$a_agent =~ s/ (I|U) *\)/)/i;   # sup I|U)
		$a_agent =~ s/ via .*//i;       # sup via proxy
		$a_agent =~ s/(\d+)\.(\d+)\.([\w-\.]+) */$1.$2.x /i; # sup Vx.x.x
		$a_agent =~ s/DigExt.*\)/)/i;
		$a_agent =~ s/;* *Nav//;
		$a_agent =~ s/\]C[^ ]+/]/;
		$a_agent =~ s/ *i\d86/ Intel/;
		$a_agent =~ s/(libwww-perl|WebZIP|GetRight)\/(\d+).*/$1\/$2/i;
		$a_agent =~ s/(\;|\,)\)+/)/g;
	} else { $a_agent = 'unknown'; }

	return ($a_rhost, $a_ruser, $a_date, $a_time, $a_file, $a_status, $a_bytes, $a_referrer, $a_agent)
}

sub sup_sub_domains {
	my ($domain, $level) = @_;
	if ($domain =~ /\d+\.\d+\.\d+\.\d+/) {
		return 'unresolved';
		if ($level eq 'xxx.dom') { return $domain; }
		if ($level eq 'dom')     { return 'unresolved'; }
	} else {
	if ($level eq 'xxx.dom') {
		$domain =~ /([\w-]+)\.(\w+)$/;
		return "$1.$2";
	}
	if ($level eq 'dom') { 
		$domain =~ /([\w-]+)\.(\w+)$/;
		return $2;
	}
}

sub try_resolv_ip {
	my ($domain) = @_;
	if ($domain !~ /\d+\.\d+\.\d+\.\d+/) { return $domain; }
	if (! $params{'ResolveIP'}) { return $domain; }
	if ($resolved_ip{$domain} ne '') { return $resolved_ip{$domain}; }
	my $packaddr = pack("C4", split(/\./, $domain));
	my $dn =  (gethostbyaddr($packaddr, 2))[0];
	if ($dn eq '') { $dn = $domain; }
	$resolved_ip{$domain} = $dn;
	return $dn;
}


sub extract_seeker_keywords {
	my($referrer) = @_;
	my($domain, $query) = split (/\?/, $referrer);
	my($ext, $dom);
	$query = &url_decode($query);
	$query = lc($query);
	$query =~ s/"|>|<|\\//g;
	$query =~ s/\s+/ /g;

	if ($domain =~ /altavista\.(\w+)/i) {
		$ext = lc($1);
		if ($query =~ /\&q=\s*([^&;]+)/i) { $seeker{"Altavista.$ext|$1"}++; }	
	} elsif ($domain =~ /\.aol\.(\w{1,4})\//i) {
		$ext = lc($1);
		if (($query =~ /search=\s*([^&;]+)/i) || ($query =~ /\&s=\s*([^&;]+)/i) || ($query =~ /\query=\s*([^&;]+)/i)) {
			$seeker{"AOL Netfind.$ext|$1"}++;
		}
	} elsif (($domain =~ /askjeeves/i) || ($domain =~ /aj\.com/i) || ($domain =~ /ask\.com/i)) {
		if ($query =~ /ask=\s*([^&;]+)/i) {	$seeker{"Askjeeves|$1"}++; }
	} elsif (($domain =~ /cnet/i) && ($query =~ /query=\s*([^&;]+)/i)) {
		$seeker{"CNET|$1"}++;
	} elsif (($domain =~ /dogpile/i) && ($query =~ /q=\s*([^&;]+)/i)) {
		$seeker{"Dog Pile|$1"}++;
	} elsif (($domain =~ /euroferret/i) && ($query =~ /p=\s*([^&;]+)/i)) {
		$seeker{"Euroferret|$1"}++;
	} elsif (($domain =~ /euroseek/i) && ($query =~ /query=\s*([^&;]+)/i)) {
		$seeker{"Euro Seek|$1"}++;
	} elsif (($domain =~ /magellan/i) || ($domain =~ /mckinley/i)) {
		if (($query =~ /search=\s*([^&;]+)/i) || ($query =~ /\&s=\s*([^&;]+)/i)) {
			$seeker{"Magellan|$1"}++;
		}
	} elsif ($domain =~ /excite/i) {
		if (($query =~ /search=\s*([^&;]+)/i) || ($query =~ /\&s=\s*([^&;]+)/i)) {
			$seeker{"Excite|$1"}++;
		}
	} elsif ($domain =~ /google/i) {
		if (($query =~ /q=\s*([^&;]+)/i) || ($query =~ /query=\s*([^&;]+)/i)) {
			$seeker{"Google|$1"}++;
		}
	} elsif (($domain =~ /goto/i) && ($query =~ /keywords=\s*([^&;]+)/i)) {
		$seeker{"GOTO|$1"}++;
	} elsif (($domain =~ /hotbot/i) && ($query =~ /mt=\s*([^&;]+)/i)) {
		$seeker{"Hot Bot|$1"}++;
	} elsif (($domain =~ /infoseek/i) || ($domain =~ /go\.com/i)) {
		my $isk = "";
		if ($query =~ /qt=\s*([^&;]+)/i) { $isk .= $1; }
		if ($query =~ /oq=\s*([^&;]+)/i) { $isk .= " ".$1; }
		$isk =~ s/^\s+//g;
		if ($isk) { $seeker{"Infoseek|$isk"}++; }
	} elsif (($domain =~ /looksmart/i) && ($query =~ /key=\s*([^&;]+)/i)) {
		$seeker{"Look Smart|$1"}++;
	} elsif (($domain =~ /lycos\./i) && ($query =~ /query=\s*([^&;]+)/i)) {
		$seeker{"Lycos|$1"}++;
	} elsif (($domain =~ /mamma/i) && ($query =~ /query=\s*([^&;]+)/i)) {
		$seeker{"Mamma|$1"}++;
	} elsif (($domain =~ /metacrawler/i) || ($domain =~ /go2net/i)) {
		if ($query =~ /general=\s*([^&;]+)/i) {
			$seeker{"Metacrawler|$1"}++;
		}
	} elsif (($domain =~ /msn/i) && ($query =~ /mt=\s*([^&;]+)/i)) {
		$seeker{"MSN|$1"}++;
	} elsif (($domain =~ /netscape/i) && ($query =~ /search=\s*([^&;]+)/i)) {
		$seeker{"Netscape|$1"}++;
	} elsif (($domain =~ /northernlight/i) && ($query =~ /qr=\s*([^&;]+)/i)) {
		$seeker{"Northernlight|$1"}++;
	} elsif (($domain =~ /planetsearch/i) && ($query =~ /text=\s*([^&;]+)/i)) {
		$seeker{"Planet Search|$1"}++;
	} elsif (($domain =~ /savvysearch/i) && ($query =~ /q=\s*([^&;]+)/i)) {
		$seeker{"Savvy Search|$1"}++;
	} elsif (($domain =~ /snap/i) && ($query =~ /keyword=\s*([^&;]+)/i)) {
		$seeker{"SNAP|$1"}++;
	} elsif ($domain =~ /webcrawler/i) {
		if (($query =~ /search=\s*([^&;]+)/i)  || ($query =~ /searchtext=\s*([^&;]+)/i)  || ($query =~ /text=\s*([^&;]+)/i)) {
			$seeker{"Webcrawler|$1"}++;
		}
	} elsif ($domain =~ /(\w+)(\.search|\.ink|\.dir)*\.yahoo\.(\w+)/i) {
		$dom = lc($1);
		$ext = lc($3);
		if ($ext =~ /com/) {
			if (($dom =~ /aunz/) || ($dom =~ /dir/) || ($dom =~ /www/) || ($dom =~ /search/) || ($dom =~ /ink/)) { $ext = ".com"; } else { $ext = ".$dom"; }
		} else {
			if ($ext ne '') { $ext = ".$ext"; } else { $ext = ".com"; }
		}
		if ($query =~ /p=\s*([^&;]+)/i) { $seeker{"Yahoo$ext|$1"}++; }
	} elsif ($domain =~ /voila\.(\w{1,4})\//i) {
		$ext = lc($1);
		if ($query =~ /kw=\s*([^&;]+)/i) { $seeker{"Voila.$ext|$1"}++; }
	} elsif (($query =~ /general=\s*([^&;]+)/i)
		  || ($query =~ /key[^=]*=\s*([^&;]+)/i)
		  || ($query =~ /query=\s*([^&;]+)/i)
		  || ($query =~ /search[^=]*=\s*([^&;]+)/i)
		  || ($query =~ /term[^=]*=\s*([^&;]+)/i)
		  || ($query =~ /text=\s*([^&;]+)/i)) 
		  {
	 		unless ($1 =~ /^[\d\.]*$/) {
				$seeker{"Other Engines|$1"}++;
			}
		}
	}
}
 
sub url_decode {
	local($_) = $_[0];
	tr/+/ /;
	s/%(..)/pack("C", hex($1))/eg;
	return $_;
}

########################################
# Date calcul subs                     #
########################################

sub date_is_sup {
	# Compar 2 date/time..
	# if date1 > date2
	my($date1, $date2) = @_;

	my($time1, $time2, @date1, @date2, $j_date1, $j_date2);
	my($hour1, $min1, $sec1, $hour2, $min2, $sec2);
	
	if ($date2 eq '') { return 1; }

	($date1, $time1) = split (/ /, $date1);
	($date2, $time2) = split (/ /, $date2);
	@date1 = split(/\//, $date1);
	@date2 = split(/\//, $date2);

	$j_date1 = &julian_day($date1[1], $date1[0], $date1[2]);
	$j_date2 = &julian_day($date2[1], $date2[0], $date2[2]);
	if ($j_date1 < $j_date2) { 
		return 0;
	}
	elsif ($j_date1 > $j_date2) { 
		return 1;
	} else {
		($hour1, $min1, $sec1) = split(/:/, $time1);
		($hour2, $min2, $sec2) = split(/:/, $time2);
		$sec1 = $hour1 * 3600 + $min1 * 60 + $sec1;
		$sec2 = $hour2 * 3600 + $min2 * 60 + $sec2;
		if ($sec1 > $sec2) { return 1; } else { return 0 }
	}
	return 0;
}

sub julian_day {
# Calcul julian day.
	my($m, $d, $y) = @_;
	my($brit_jd) = 2361222;
	my($ya, $c, $jd);

	$y = (localtime(time))[5] + 1900  if ($y eq '');

	if ($m > 2) {
		$m -= 3;
	} else {
		$m += 9;
		--$y;
	}
	$c = int($y/100);
	$ya = $y - (100 * $c);
	$jd =  int((146097 * $c) / 4) +
		   int((1461 * $ya) / 4) +
		   int((153 * $m + 2) / 5) +
		   $d + 1721119;
	warn("warning:  pre-dates British use of Gregorian calendar\n")
		if ($jd < $brit_jd);
	return $jd;
}

sub jd2wkday {
	return ($_[0] + 1) % 7;
}

sub date2wkday {
	my($m, $d, $y) = @_;
	return &jd2wkday(&julian_day($m, $d, $y));
}

